@@ -869,9 +869,9 @@ xsltApplyTemplatesComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	if (prop == NULL) {
 	    if (style != NULL) style->errors++;
 	} else {
-	    comp->mode = prop;
+	    comp->mode = xmlDictLookup(style->dict, prop, -1);
 	    if (URI != NULL) {
-		comp->modeURI = xmlStrdup(URI);
+		comp->modeURI = xmlDictLookup(style->dict, URI, -1);
 	    } else {
 		comp->modeURI = NULL;
 	    }
@@ -1057,7 +1057,7 @@ xsltVariableComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	    comp->name = prop;
 	    comp->has_name = 1;
 	    if (URI != NULL) {
-		comp->ns = xmlStrdup(URI);
+		comp->ns = xmlDictLookup(style->dict, URI, -1);
 		comp->has_ns = 1;
 	    } else {
 		comp->has_ns = 0;
