@@ -319,8 +319,10 @@ xsltFreeTemplate(xsltTemplatePtr template) {
     if (template->match) xmlFree(template->match);
     if (template->name) xmlFree(template->name);
     if (template->nameURI) xmlFree(template->nameURI);
+/*
     if (template->mode) xmlFree(template->mode);
     if (template->modeURI) xmlFree(template->modeURI);
+ */
     if (template->inheritedNs) xmlFree(template->inheritedNs);
     memset(template, -1, sizeof(xsltTemplate));
     xmlFree(template);
@@ -372,6 +374,7 @@ xsltNewStylesheet(void) {
     cur->exclPrefixTab = NULL;
     cur->extInfos = NULL;
     cur->extrasNr = 0;
+    cur->internalized = 1;
     cur->dict = xmlDictCreate();
 #ifdef WITH_XSLT_DEBUG
     xsltGenericDebug(xsltGenericDebugContext,
@@ -426,7 +429,7 @@ xsltAllocateExtraCtxt(xsltTransformContextPtr ctxt)
 	    for (i = 0;i < ctxt->extrasMax;i++) {
 		ctxt->extras[i].info = NULL;
 		ctxt->extras[i].deallocate = NULL;
-		ctxt->extras[i].val = NULL;
+		ctxt->extras[i].val.ptr = NULL;
 	    }
 
 	} else {
@@ -445,7 +448,7 @@ xsltAllocateExtraCtxt(xsltTransformContextPtr ctxt)
 	    for (i = ctxt->extrasNr;i < ctxt->extrasMax;i++) {
 		ctxt->extras[i].info = NULL;
 		ctxt->extras[i].deallocate = NULL;
-		ctxt->extras[i].val = NULL;
+		ctxt->extras[i].val.ptr = NULL;
 	    }
 	}
     }
@@ -1178,6 +1181,13 @@ xsltParseStylesheetExcludePrefix(xsltStylesheetPtr style, xmlNodePtr cur) {
 static void
 xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
     xmlNodePtr delete;
+    int internalize = 0;
+
+    if ((style != NULL) && (cur != NULL) && (cur->doc != NULL) &&
+        (style->dict != NULL) && (cur->doc->dict == style->dict))
+	internalize = 1;
+    else
+        style->internalized = 0;
 
     /*
      * This content comes from the stylesheet
@@ -1198,6 +1208,34 @@ xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
 	if (cur->type == XML_ELEMENT_NODE) {
 	    int exclPrefixes;
 
+	    /*
+	     * Internalize attributes values.
+	     */
+	    if ((internalize) && (cur->properties != NULL)) {
+	        xmlAttrPtr prop = cur->properties;
+		xmlNodePtr txt;
+
+		while (prop != NULL) {
+		    txt = prop->children;
+		    if ((txt != NULL) && (txt->type == XML_TEXT_NODE) &&
+		        (txt->content != NULL) &&
+			(!xmlDictOwns(style->dict, txt->content))) {
+			xmlChar *old = (xmlChar *) txt->content;
+
+			/*
+			 * internalize the text string, goal is to speed
+			 * up operations and minimize used space by compiled
+			 * stylesheets.
+			 */
+			txt->content = (xmlChar *)
+			               xmlDictLookup(style->dict, old, -1);
+			if (old != txt->content)
+			    xmlFree(old);
+		    }
+		    prop = prop->next;
+		}
+	    }
+
 	    exclPrefixes = xsltParseStylesheetExcludePrefix(style, cur);
 	    if (IS_XSLT_ELEM(cur)) {
 		xsltStylePreCompute(style, cur);
@@ -1264,6 +1302,17 @@ xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
 		if (xmlNodeGetSpacePreserve(cur) != 1) {
 		    delete = cur;
 		}
+	    } else if ((cur->content != NULL) && (internalize) &&
+	               (!xmlDictOwns(style->dict, cur->content))) {
+		xmlChar *old = (xmlChar *) cur->content;
+
+		/*
+		 * internalize the text string, goal is to speed
+		 * up operations and minimize used space by compiled
+		 * stylesheets.
+		 */
+		cur->content = (xmlChar *) xmlDictLookup(style->dict, old, -1);
+		xmlFree(old);
 	    }
 	} else if ((cur->type != XML_ELEMENT_NODE) &&
 		   (cur->type != XML_CDATA_SECTION_NODE)) {
@@ -1473,8 +1522,28 @@ xsltParseTemplateContent(xsltStylesheetPtr style, xmlNodePtr templ) {
 		     * replace xsl:text by the list of childs
 		     */
 		    if (text == NULL) {
+		        int internalize = 0;
+			if ((style != NULL) && (text != NULL) &&
+			    (text->doc != NULL) && (style->dict != NULL) &&
+			    (text->doc->dict == style->dict))
+			    internalize = 1;
+			else
+			    style->internalized = 0;
+
 			text = cur->children;
 			while (text != NULL) {
+			    if ((text->content != NULL) &&
+			        (!xmlDictOwns(style->dict, text->content))) {
+				xmlChar *old = (xmlChar *) text->content;
+
+				/*
+				 * internalize the text string
+				 */
+				text->content = (xmlChar *)
+					xmlDictLookup(style->dict, old, -1);
+				xmlFree(old);
+			    }
+
 			    next = text->next;
 			    xmlUnlinkNode(text);
 			    xmlAddPrevSibling(cur, text);
@@ -1713,15 +1782,14 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
 	    if (URI != NULL)
 		modeURI = xmlStrdup(URI);
 	}
-	ret->mode = mode;
-	ret->modeURI = modeURI;
+	ret->mode = xmlDictLookup(style->dict, mode, -1);
+	ret->modeURI = xmlDictLookup(style->dict, modeURI, -1);
 #ifdef WITH_XSLT_DEBUG_PARSING
 	xsltGenericDebug(xsltGenericDebugContext,
 	     "xsltParseStylesheetTemplate: mode %s\n", mode);
 #endif
-    } else {
-	mode = NULL;
-	modeURI = NULL;
+        if (mode != NULL) xmlFree(mode);
+	if (modeURI != NULL) xmlFree(modeURI);
     }
     prop = xsltGetNsProp(template, (const xmlChar *)"match", XSLT_NAMESPACE);
     if (prop != NULL) {
@@ -1768,7 +1836,10 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
 		ret->nameURI = NULL;
 	    cur = ret->next;
 	    while (cur != NULL) {
-	        if (xmlStrEqual(cur->name, prop)) {
+	        if ((URI != NULL && xmlStrEqual(cur->name, prop) &&
+				xmlStrEqual(cur->nameURI, URI) ) ||
+		    (URI == NULL && cur->nameURI == NULL &&
+				xmlStrEqual(cur->name, prop))) {
 		    xsltTransformError(NULL, style, template,
 		        "xsl:template: error duplicate name '%s'\n", prop);
 		    style->errors++;
@@ -1785,7 +1856,7 @@ xsltParseStylesheetTemplate(xsltStylesheetPtr style, xmlNodePtr template) {
     xsltParseTemplateContent(style, template);
     ret->elem = template;
     ret->content = template->children;
-    xsltAddTemplate(style, ret, mode, modeURI);
+    xsltAddTemplate(style, ret, ret->mode, ret->modeURI);
 
 error:
     for (;exclPrefixes > 0;exclPrefixes--)
