@@ -23,28 +23,28 @@ extern "C" {
  *
  * the version string like "1.2.3"
  */
-#define LIBXSLT_DOTTED_VERSION "1.1.12"
+#define LIBXSLT_DOTTED_VERSION "1.1.13"
 
 /**
  * LIBXSLT_VERSION:
  *
  * the version number: 1.2.3 value is 1002003
  */
-#define LIBXSLT_VERSION 10112
+#define LIBXSLT_VERSION 10113
 
 /**
  * LIBXSLT_VERSION_STRING:
  *
  * the version number string, 1.2.3 value is "1002003"
  */
-#define LIBXSLT_VERSION_STRING "10112"
+#define LIBXSLT_VERSION_STRING "10113"
 
 /**
- * LIBXSLT_VERSION_EXTRA
+ * LIBXSLT_VERSION_EXTRA:
  *
  * extra version information, used to show a CVS compilation
  */
-#define LIBXML_VERSION_EXTRA "-CVS964"
+#define LIBXSLT_VERSION_EXTRA "-CVS999"
 
 /**
  * WITH_XSLT_DEBUG:
@@ -57,6 +57,18 @@ extern "C" {
 #define WITH_XSLT_DEBUG
 #endif
 
+/**
+ * WITH_MODULES:
+ *
+ * Whether module support is configured into libxslt
+ */
+#if 1
+#ifndef WITH_MODULES
+#define WITH_MODULES
+#endif
+#define LIBXSLT_PLUGINS_PATH() getenv("LIBXSLT_PLUGINS_PATH")
+#endif
+
 #if 0
 /**
  * DEBUG_MEMORY:
